"""Compare the fraction of support vectors for RLSVM2"""

import sys
from pprint import pprint
from optwok.expttools import apply_to_combination as atc
from mlexpt import MLExpt
from pylab import show, figure, savefig
from pylab import boxplot, xticks, yticks, grid
from pylab import axis, xlabel, ylabel, legend, title
import pylab
from numpy import zeros, arange, max

def configure_from_file(filename, algo):
    """Configure the experiment"""
    config = dict()
    execfile(filename,{'algo':algo},config)
    config['algo'] = algo
    return config

def plot_found_feat(algo, datasets, num_perm, acc, toprint=True):
    """
    Collect and plot the results on the
    false positive and false negative features.
    """
    pylab.rcParams['font.size'] = 22.0
    num_examples = [100]
    num_noise_feats = [0, 10, 100, 200]
    fp = zeros((num_perm, len(num_noise_feats)*len(num_examples)))
    fn = zeros((num_perm, len(num_noise_feats)*len(num_examples)))
    for (idx, nnf) in enumerate(num_noise_feats):
        for (nix, num_ex) in enumerate(num_examples):
            dataname = 'twoballs_n=%d_10:%d_rcn=0.0_all' % (num_ex, nnf)
            fp[:,idx*len(num_examples)+nix] = acc[dataname]['false_positive']
            fn[:,idx*len(num_examples)+nix] = acc[dataname]['false_negative']
    
    figure()
    boxplot(fp)
    title(algo)
    ylabel('false positive features')
    xlabel('number of noisy features')
    xticks(arange(len(num_noise_feats))+1, num_noise_feats)
    if max(fp) > 20:
        print 'Warning: axis truncation!'
    axis([0.5, 4.5, 0, 20])
    if toprint:
        savefig('feat_false_positive_%s.png' % algo)
    figure()
    boxplot(fn)
    title(algo)
    ylabel('false negative features')
    xlabel('number of noisy features')
    xticks(arange(len(num_noise_feats))+1, num_noise_feats)
    axis([0.5, 4.5, 0, 10])
    if toprint:
        savefig('feat_false_negative_%s.png' % algo)
    if not toprint:
        show()

def main(config):
    """Run the experiment for rlsvm"""
    print '================================'
    print 'Collecting results: %s' % config['algo']
    print '================================'
    search_param = {'reg_param': config['reg_params'],
                    'kparam_idx': 0,
                    }
    search_param['mu'] = [1.0]
    expt = MLExpt(config)

    best_param = expt.find_best_param(config['datasets'], config['perm_idx'],
                                 config['validation_conf']['num_cv'], search_param)
    acc = expt.compute_found_feat(config['datasets'], config['perm_idx'], best_param)
    plot_found_feat(algo, config['datasets'], len(config['perm_idx']), acc)

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print 'Usage: python %s configure.py' % sys.argv[0]
        exit(1)
    classifiers = ['OneNormSVM','CapOneNormSVM','ZeroNormSVM','RLSVM1','RLSVMC1','RLSVM0']
    for algo in classifiers:
        filename = sys.argv[1]
        settings = configure_from_file(filename, algo)
        main(settings)
